package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.FlightSplitTaskBo;
import org.eco.als.domain.vo.FlightSplitTaskImportVo;
import org.eco.als.domain.vo.FlightSplitTaskVo;
import org.eco.als.service.IFlightSplitTaskService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 飞行分解任务Controller
 *
 * @author wgk
 * @date 2024-07-24
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/flightSplitTask")
public class FlightSplitTaskController extends BaseController {
    @Resource
    private IFlightSplitTaskService flightSplitTaskService;

    /**
     * 查询飞行分解任务列表
     */
    @SaCheckPermission("als:flightSplitTask:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FlightSplitTaskVo>> list(FlightSplitTaskBo flightSplitTaskBo) {
        return CommonResult.success(flightSplitTaskService.selectPage(flightSplitTaskBo));
    }

    /**
     * 导出飞行分解任务列表
     */
    @SaCheckPermission("als:flightSplitTask:export")
    @Log(title = "飞行分解任务", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FlightSplitTaskBo flightSplitTaskBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FlightSplitTaskVo> list = flightSplitTaskService.selectList(flightSplitTaskBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        flightSplitTaskService.asyncExport(list, "飞行分解任务", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:flightSplitTask:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "飞行分解任务", FlightSplitTaskImportVo.class, response);
    }

    /**
     * 导入飞行分解任务列表
     */
    @Log(title = "飞行分解任务", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:flightSplitTask:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        flightSplitTaskService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取飞行分解任务详细信息
     */
    @SaCheckPermission("als:flightSplitTask:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FlightSplitTaskVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(flightSplitTaskService.selectById(id));
    }

    /**
     * 新增飞行分解任务
     */
    @SaCheckPermission("als:flightSplitTask:add")
    @Log(title = "飞行分解任务", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FlightSplitTaskBo flightSplitTaskBo) {
        boolean inserted = flightSplitTaskService.insert(flightSplitTaskBo);
        if (!inserted) {
            return CommonResult.fail("新增飞行分解任务记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改飞行分解任务
     */
    @SaCheckPermission("als:flightSplitTask:edit")
    @Log(title = "飞行分解任务", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FlightSplitTaskBo flightSplitTaskBo) {
        boolean updated = flightSplitTaskService.update(flightSplitTaskBo);
        if (!updated) {
            return CommonResult.fail("修改飞行分解任务记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除飞行分解任务
     */
    @SaCheckPermission("als:flightSplitTask:remove")
    @Log(title = "飞行分解任务", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = flightSplitTaskService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除飞行分解任务记录失败!");
        }
        return CommonResult.success();
    }
}
