package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.FlightTask;
import org.eco.als.domain.bo.FlightTaskBo;
import org.eco.als.domain.vo.FlightTaskImportVo;
import org.eco.als.domain.vo.FlightTaskVo;
import org.eco.als.listener.FlightTaskImportListener;
import org.eco.als.mapper.FlightTaskMapper;
import org.eco.als.service.IFlightTaskService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.FlightTaskTableDef.FLIGHT_TASK;

/**
 * 飞行任务Service业务层处理
 *
 * @author wgk
 * @date 2024-07-24
 */
@Service
@Slf4j
public class FlightTaskServiceImpl extends BaseServiceImpl<FlightTaskMapper, FlightTask> implements IFlightTaskService {
    @Resource
    private FlightTaskMapper flightTaskMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FLIGHT_TASK);
    }

    private QueryWrapper buildQueryWrapper(FlightTaskBo flightTaskBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FLIGHT_TASK.SORTIE_NO.eq
            (flightTaskBo.getSortieNo()));
        queryWrapper.and(FLIGHT_TASK.AIRCRAFT_MODEL.eq
            (flightTaskBo.getAircraftModel()));
        queryWrapper.and(FLIGHT_TASK.AIRCRAFT_ID.eq
            (flightTaskBo.getAircraftId()));
        queryWrapper.and(FLIGHT_TASK.START_TIME.eq
            (flightTaskBo.getStartTime()));
        queryWrapper.and(FLIGHT_TASK.END_TIME.eq
            (flightTaskBo.getEndTime()));
        queryWrapper.and(FLIGHT_TASK.TRAIN_EXERCISE.eq
            (flightTaskBo.getTrainExercise()));
        queryWrapper.and(FLIGHT_TASK.STATUS.eq
            (flightTaskBo.getStatus()));

        return queryWrapper;
    }

    /**
     * 查询飞行任务
     *
     * @param id 飞行任务主键
     * @return 飞行任务
     */
    @Override
    public FlightTaskVo selectById(Long id) {
        return this.getOneAs(query().where(FLIGHT_TASK.ID.eq(id)), FlightTaskVo.class);

    }


    /**
     * 查询飞行任务列表
     *
     * @param flightTaskBo 飞行任务Bo
     * @return 飞行任务集合
     */
    @Override
    public List<FlightTaskVo> selectList(FlightTaskBo flightTaskBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(flightTaskBo);
        return this.listAs(queryWrapper, FlightTaskVo.class);
    }

    /**
     * 分页查询飞行任务列表
     *
     * @param flightTaskBo 飞行任务Bo
     * @return 分页飞行任务集合
     */
    @Override
    public PageResult<FlightTaskVo> selectPage(FlightTaskBo flightTaskBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(flightTaskBo);
        Page<FlightTaskVo> page = this.pageAs(PageQuery.build(), queryWrapper, FlightTaskVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增飞行任务
     *
     * @param flightTaskBo 飞行任务Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FlightTaskBo flightTaskBo) {
        FlightTask flightTask = MapstructUtils.convert(flightTaskBo, FlightTask.class);

        return this.save(flightTask);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增飞行任务，前台提供主键值，一般用于导入的场合
     *
     * @param flightTaskBo 飞行任务Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FlightTaskBo flightTaskBo) {
        FlightTask flightTask = MapstructUtils.convert(flightTaskBo, FlightTask.class);


        return flightTaskMapper.insertWithPk(flightTask) > 0;//前台传来主键值
    }

    /**
     * 修改飞行任务
     *
     * @param flightTaskBo 飞行任务Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FlightTaskBo flightTaskBo) {
        FlightTask flightTask = MapstructUtils.convert(flightTaskBo, FlightTask.class);
        if (ObjectUtil.isNotNull(flightTask) && ObjectUtil.isNotNull(flightTask.getId())) {
            return this.updateById(flightTask);
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, FlightTaskImportVo.class, new FlightTaskImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<FlightTaskVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FlightTaskVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除飞行任务
     *
     * @param ids 需要删除的飞行任务主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
